-- Copyright 2024-2025 by Todd Hundersmarck (ThundR) 
-- All Rights Reserved

local thModName = g_currentModName
local thModPath = g_currentModDirectory
source(thModPath .. "scripts/utilities/THUtils.lua")
source(thModPath .. "scripts/utilities/THDebugUtil.lua")
THDefaultTypes = {}
local THDefaultTypes_mt = THUtils.createClass(THDefaultTypes)
THDefaultTypes.DATA_KEY = "thDefaultTypes"
THDefaultTypes.XML_KEY = THDefaultTypes.DATA_KEY
function THDefaultTypes.new(customMt)
    customMt = customMt or THDefaultTypes_mt
    if THUtils.argIsValid(type(customMt) == "table", "customMt", customMt) then
        local self = setmetatable({}, customMt)
        self.isServer = g_server ~= nil
        self.isClient = g_client ~= nil
        self.modName = thModName
        self.modPath = thModPath
        self.dataKey = THDefaultTypes.DATA_KEY
        self.xmlKey = THDefaultTypes.XML_KEY
        self.fruitTypeManager = g_fruitTypeManager
        self.defaultTypes = {}
        return self
    end
end
function THDefaultTypes.clearDefaultTypeData(self, typeKey)
    if typeKey ~= nil
        and THUtils.argIsValid(type(typeKey) == "string", "typeKey", typeKey)
    then
        local typeData = self.defaultTypes[typeKey]
        if typeData ~= nil then
            if typeData.dataXMLFile ~= nil then
                THUtils.deleteXMLFile(typeData.dataXMLFile)
            end
            THUtils.clearTable(typeData)
            self.defaultTypes[typeKey] = nil
            return true
        end
    end
    return false
end
function THDefaultTypes.loadDefaultTypeData(self, mapXMLFile, typeKey, missionInfo, customEnv, baseDirectory)
    if THUtils.argIsValid(type(typeKey) == "string", "typeKey", typeKey)
        and THUtils.argIsValid(type(missionInfo) == "table", "missionInfo", missionInfo)
        and THUtils.argIsValid(customEnv == nil or type(customEnv) == THValueType.STRING, "customEnv", customEnv)
        and THUtils.argIsValid(baseDirectory == nil or type(baseDirectory) == "string", "baseDirectory", baseDirectory)
    then
        local mapDesc = missionInfo.map
        local dataXMLKey = string.format("map.%s", typeKey)
        if mapDesc ~= nil and mapDesc.isModMap then
            if customEnv == nil or customEnv == "" then
                customEnv = mapDesc.customEnvironment
            end
            if baseDirectory == nil or baseDirectory == "" then
                baseDirectory = mapDesc.baseDirectory
            end
            if customEnv ~= nil and baseDirectory ~= nil then
                local typeData = {
                    mapXMLFile = mapXMLFile,
                    mapModName = customEnv,
                    mapModPath = baseDirectory,
                    missionInfo = missionInfo,
                    isDefaultLoading = false
                }
                local thMapSettings = THUtils.loadMapSettings()
                local isDefaultTypes = false
                local defaultTypesKey = "#" .. self.dataKey
                if thMapSettings ~= nil then
                    local xmlName = string.format("THMapSettingsXML_%s", typeKey)
                    local dataXMLFile, _, isMapSettings = THUtils.loadMapXMLFile(mapDesc, xmlName, nil, false, true, true)
                    if dataXMLFile ~= nil then
                        if isMapSettings then
                            isDefaultTypes = THUtils.getXMLValue(dataXMLFile, XMLValueType.BOOL, dataXMLKey, defaultTypesKey, false)
                            if isDefaultTypes then
                                typeData.dataXMLFile = dataXMLFile
                                typeData.dataModName = thMapSettings.modName
                                typeData.dataModPath = thMapSettings.modPath
                            end
                        end
                        if typeData.dataXMLFile == nil then
                            THUtils.deleteXMLFile(dataXMLFile)
                        end
                    end
                end
                if not isDefaultTypes then
                    isDefaultTypes = THUtils.getXMLValue(typeData.mapXMLFile, XMLValueType.BOOL, dataXMLKey, defaultTypesKey, false)
                end
                if isDefaultTypes then
                    return typeData
                end
            end
        end
    end
end
function THDefaultTypes.inj_loadDataFromMapXML(self, superFunc, mapXMLFile, typeKey, ...)
    local skipLoad = false
    if mapXMLFile ~= nil and typeKey ~= nil then
        THUtils.call(function()
            local typeData = self.defaultTypes[typeKey]
            if typeData ~= nil and mapXMLFile == typeData.mapXMLFile
                and not typeData.isDefaultLoading
            then
                skipLoad = true
            end
        end)
    end
    if skipLoad then
        return true
    end
    return superFunc(mapXMLFile, typeKey, ...)
end
function THDefaultTypes.inj_loadFillTypeMapData(self, superFunc, manager, xmlFile, missionInfo, baseDirectory, ...)
    if xmlFile ~= nil and missionInfo ~= nil and baseDirectory ~= nil then
        THUtils.call(function()
            self.defaultTypes.fillTypes = self:loadDefaultTypeData(xmlFile, "fillTypes", missionInfo, nil, baseDirectory)
        end)
    end
    local function appendFunc(...)
        THUtils.call(function()
            self:clearDefaultTypeData("fillTypes")
        end)
        return ...
    end
    return appendFunc(superFunc(manager, xmlFile, missionInfo, baseDirectory, ...))
end
function THDefaultTypes.inj_loadFillTypes(self, superFunc, manager, xmlFile, baseDirectory, isBaseType, modName, ...)
    if isBaseType == true then
        local typeData = self.defaultTypes.fillTypes
        if typeData ~= nil and not typeData.isDefaultLoading
            and typeData.mapXMLFile ~= nil and typeData.missionInfo ~= nil
        then
            typeData.isDefaultLoading = true
            local function appendFunc(rSuccess, ...)
                typeData.isDefaultLoading = false
                if rSuccess then
                    THUtils.displayMsg("Default fill types loaded from map xml")
                end
                return rSuccess, ...
            end
            local mapXMLFile = typeData.mapXMLFile
            local mapModName = typeData.mapModName
            local mapModPath = typeData.mapModPath
            if typeData.dataXMLFile ~= nil then
                mapXMLFile = typeData.dataXMLFile
                mapModName = typeData.mapModName
                mapModPath = typeData.dataModPath
            end
            return appendFunc(XMLUtil.loadDataFromMapXML(mapXMLFile, "fillTypes", mapModPath, manager, superFunc, mapModPath, true, mapModName))
        end
    end
    return superFunc(manager, xmlFile, baseDirectory, isBaseType, modName, ...)
end
function THDefaultTypes.inj_addFillType(self, superFunc, manager, desc, ...)
    local skipAdd = false
    if desc ~= nil then
        THUtils.call(function()
            if desc.name ~= nil then
                local otherDesc = manager:getFillTypeByName(desc.name)
                if otherDesc ~= nil then
                    skipAdd = true
                    THUtils.errorMsg(false, "Fill type %q already registered at index %s, skipping addition...", otherDesc.name, otherDesc.index)
                end
            end
        end)
    end
    if skipAdd then
        return false
    end
    return superFunc(manager, desc, ...)
end
function THDefaultTypes.inj_loadFruitTypeMapData(self, superFunc, manager, xmlFile, missionInfo, baseDirectory, ...)
    if xmlFile ~= nil and missionInfo ~= nil and baseDirectory ~= nil then
        THUtils.call(function()
            self.defaultTypes.fruitTypes = self:loadDefaultTypeData(xmlFile, "fruitTypes", missionInfo, nil, baseDirectory)
        end)
    end
    local function appendFunc(...)
        THUtils.call(function()
            self:clearDefaultTypeData("fruitTypes")
        end)
        return ...
    end
    return appendFunc(superFunc(manager, xmlFile, missionInfo, baseDirectory, ...))
end
function THDefaultTypes.inj_loadFruitTypes(self, superFunc, manager, xmlFile, missionInfo, baseDirectory, isBaseType, ...)
    if isBaseType == true then
        local typeData = self.defaultTypes.fruitTypes
        if typeData ~= nil and not typeData.isDefaultLoading
            and typeData.mapXMLFile ~= nil and typeData.missionInfo ~= nil
        then
            typeData.isDefaultLoading = true
            local function appendFunc(rSuccess, ...)
                typeData.isDefaultLoading = false
                if rSuccess then
                    THUtils.displayMsg("Default fruit types loaded from map xml")
                end
                return rSuccess, ...
            end
            local mapXMLFile = typeData.mapXMLFile
            local mapModPath = typeData.mapModPath
            if typeData.dataXMLFile ~= nil then
                mapXMLFile = typeData.dataXMLFile
                mapModPath = typeData.dataModPath
            end
            return appendFunc(XMLUtil.loadDataFromMapXML(mapXMLFile, "fruitTypes", mapModPath, manager, superFunc, typeData.missionInfo, mapModPath, true))
        end
    end
    return superFunc(manager, xmlFile, missionInfo, baseDirectory, isBaseType, ...)
end
function THDefaultTypes.inj_loadFoliageFromXMLFile(self, superFunc, desc, filename, ...)
    local isFruitTypeSwapped = false
    if desc ~= nil and filename ~= nil then
        THUtils.call(function()
            if fileExists(filename) then
                local fruitXMLFile = THUtils.loadXMLFile("THFruitType", filename, nil, false)
                if THUtils.getNoNil(fruitXMLFile, 0) > 0 then
                    local fruitTypeName = getXMLString(fruitXMLFile, "foliageType.fruitType#name")
                    if fruitTypeName ~= nil then
                        fruitTypeName = string.upper(fruitTypeName)
                        local otherDesc = self.fruitTypeManager:getFruitTypeByName(fruitTypeName)
                        if otherDesc ~= nil and otherDesc.index ~= desc.index then
                            THUtils.errorMsg(false, "Fruit type %q already registered at index %s, updating existing entry...", fruitTypeName, otherDesc.index)
                            desc = otherDesc
                            isFruitTypeSwapped = true
                        end
                    end
                    THUtils.deleteXMLFile(fruitXMLFile)
                end
            end
        end)
    end
    local function appendFunc(rSuccess, ...)
        if rSuccess then
            THUtils.call(function()
                if isFruitTypeSwapped then
                    rSuccess = false
                end
            end)
        end
        return rSuccess, ...
    end
    return appendFunc(superFunc(desc, filename, ...))
end
THUtils.call(function()
    local self = THDefaultTypes.new()
    if self ~= nil then
        _G.g_thDefaultTypes = self
        THUtils.setFunctionHook("XMLUtil", "loadDataFromMapXML", false, false, self, THDefaultTypes.inj_loadDataFromMapXML)
        THUtils.setFunctionHook("FillTypeManager", "loadMapData", false, false, self, THDefaultTypes.inj_loadFillTypeMapData)
        THUtils.setFunctionHook("FillTypeManager", "loadFillTypes", false, false, self, THDefaultTypes.inj_loadFillTypes)
        THUtils.setFunctionHook("FillTypeManager", "addFillType", false, false, self, THDefaultTypes.inj_addFillType)
        THUtils.setFunctionHook("FruitTypeManager", "loadMapData", false, false, self, THDefaultTypes.inj_loadFruitTypeMapData)
        THUtils.setFunctionHook("FruitTypeManager", "loadFruitTypes", false, false, self, THDefaultTypes.inj_loadFruitTypes)
        THUtils.setFunctionHook("FruitTypeDesc", "loadFromFoliageXMLFile", false, false, self, THDefaultTypes.inj_loadFoliageFromXMLFile)
    end
end)